<?php 

/*******************************************************************************
 *  Provide information to drupal about the node types that we're creating
*  in this module
*/
function tripal_analysis_interpro_node_info() {
  $nodes = array();
  $nodes['chado_analysis_interpro'] = array(
      'name' => t('Analysis: Interpro'),
      'base' => 'chado_analysis_interpro',
      'description' => t('An interpro analysis from the chado database'),
      'has_title' => FALSE,
      'title_label' => t('Analysis: Interpro'),
      'has_body' => FALSE,
      'body_label' => t('Interpro Analysis Description'),
      'locked' => TRUE
  );
  return $nodes;
}

/*******************************************************************************
 *  The following function proves access control for users trying to
*  perform actions on data managed by this module
*/
function chado_analysis_interpro_access($op, $node, $account) {
  if ($op == 'create') {
    if (!user_access('create chado_analysis_interpro content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'update') {
    if (!user_access('edit chado_analysis_interpro content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'delete') {
    if (!user_access('delete chado_analysis_interpro content', $account)) {
      return FALSE;
    }
  }
  if ($op == 'view') {
    if (!user_access('access chado_analysis_interpro content', $account)) {
      return FALSE;
    }
  }
  return NULL;
}

/*******************************************************************************
 *  Provide a Interpro Analysis form
*/
function chado_analysis_interpro_form($node, $form_state) {

  // add in the default fields for the analysis
  $form = chado_analysis_form($node, $form_state);
  // set the fedaults
  $interprofile = '';
  $interprojob = '';
  $parsego = '';
  $interproparameters = '';
  $query_re = '';
  $query_type = '';
  $query_uniquename = '';
  
  if (property_exists ( $node, 'analysis' )) {
    $analysis = $node->analysis;
    $interprofile = $analysis->tripal_analysis_interpro->interprofile;
    $interproparameters = $analysis->tripal_analysis_interpro->interproparameters;
    $parsego = $analysis->tripal_analysis_interpro->parsego;
    $query_re = $analysis->tripal_analysis_interpro->query_re;
    $query_type = $analysis->tripal_analysis_interpro->query_type;
    $query_uniquename = $analysis->tripal_analysis_interpro->query_uniquename;
  }

  $moreSettings ['interpro'] = 'Interpro Settings';
  $form['interpro'] = array(
      '#title' => t('Interpro Settings'),
      '#type' => 'fieldset',
      '#description' => t('Specific Settings for Interpro Analysis.'),
      '#collapsible' => TRUE,
      '#attributes' => array('id' => 'interpro-extra-settings'),
      '#weight' => 11
  );
  $form['interpro']['interprofile'] = array(
      '#title' => t('InterProScan XML File/Directory: (if you input a directory without the tailing slash, all xml files in the directory will be loaded)'),
      '#type' => 'textfield',
      '#description' => t('Please provide the full path to the XML output file generated by InterProScan or a directory containing multiple XML files.'),
      '#default_value' => $interprofile,
  );
  $form['interpro']['interprojob'] = array(
      '#type' => 'checkbox',
      '#title' => t('Submit a job to parse the InterProScan XML file(s)'),
      '#description' => t('Note: features associated with the interpro results must '.
          'exist in chado before parsing the file. Otherwise, interpro '.
          'results that cannot be linked to a feature will not '.
          'be imported.  The feature name must be unique'),
      '#default_value' => $interprojob,
      '#attributes' => array(
          'onclick' => 'return isSubmittingJob(this)'
      )
  );
  $form['interpro']['parsego'] = array(
      '#type' => 'checkbox',
      '#title' => t('Load GO terms to the database'),
      '#description' => t('Check the box to load GO terms to chado database'),
      '#default_value' => $parsego
  );
  $form['interpro']['interproparameters'] = array(
      '#title' => t('Parameters'),
      '#type' => 'textfield',
      '#description' => t('The parameters used when running the InterProScan analysis.'),
      '#default_value' => $interproparameters,
  );

  $form['interpro']['query_re'] = array(
      '#title' => t('Query Name RE'),
      '#type' => 'textfield',
      '#description' => t('Enter the regular expression that will extract the '.
          'feature name from the query line in the interpro results. This option is '.
          'is only required when the query does not identically match a feature '.
          'in the database. By default, the parser will try to match results to '.
          'a feature in Chado using the feature name.  Select the check box below'.
          'to match against the unique name if needed.'),
      '#default_value' => $query_re,
  );

  $form['interpro']['query_uniquename'] = array(
      '#title' => t('Use Unique Name'),
      '#type' => 'checkbox',
      '#description' => t('Select this checkbox if the query name in the results file matches the unique name of the feature. '),
      '#default_value' => $query_uniquename,
  );

  $form['interpro']['query_type'] = array(
      '#title' => t('Query Type'),
      '#type' => 'textfield',
      '#description' => t('Please enter the Sequence Ontology term (e.g. contig, polypeptide, mRNA) that describes '.
          'the query sequences in the InterProScan XML results file(s).  This is only necessary if two '.
          'or more sequences have the same name.'),
      '#default_value' => $query_type,
  );
  return $form;
}
/**
 *
 */
function chado_analysis_interpro_validate($node, &$form, &$form_state) {
  
  // use the analysis parent to validate the node
  tripal_analysis_validate($node, $form, $form_state);
  
  // trim character fields
  $node->interprofile      = trim($node->interprofile);
  $node->query_uniquename  = trim($node->query_uniquename);
  $node->query_type        = trim($node->query_type);
  
  if($node->interprofile and !is_readable($node->interprofile)) {
    form_set_error('interprofile', "Can not read InterPro XML output file: '$node->interprofile'. Please check file permissions or typos in the file path.");
  }
  
}
/**
 *
 *
 */
function chado_analysis_interpro_load($nodes) {

  // load the default set of analysis fields
  chado_analysis_load($nodes);

  foreach ($nodes as $nid => $node) {
    // create some variables for easier lookup
    $analysis = $node->analysis;
    $analysis_id = $analysis->analysis_id;

    $interpro_settings  = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_settings');
    $interprofile      = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_interprofile');
    $interproparameters= tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_interproparameters');
    $parsego           = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_parsego');
    $query_re          = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_query_re');
    $query_type        = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_query_type');
    $query_uniquename  = tripal_analysis_get_property($analysis->analysis_id, 'analysis_interpro_query_uniquename');

    $analysis->tripal_analysis_interpro = new stdClass;
    $analysis->tripal_analysis_interpro->interprofile      = $interprofile->value;
    $analysis->tripal_analysis_interpro->interproparameters= $interproparameters->value;
    $analysis->tripal_analysis_interpro->parsego           = $parsego->value;
    $analysis->tripal_analysis_interpro->query_re          = $query_re->value;
    $analysis->tripal_analysis_interpro->query_type        = $query_type->value;
    $analysis->tripal_analysis_interpro->query_uniquename  = $query_uniquename->value;

    // if there is an old style 'interpro_settings' array, then break these out for
    // use in the new format
    if (count($interpro_settings)>0) {
      $prop_values = explode("|", $interpro_settings->value);
      $analysis->tripal_analysis_interpro->interprofile       = $prop_values[0];
      $analysis->tripal_analysis_interpro->interproparameters = $prop_values[1];
    }
    
    $nodes[$nid]->analysis = $analysis;
  }
}
/**
 *
 */
function chado_analysis_interpro_insert($node) {
  
  // insert the analysis. If the analysis already exist then this
  // call will link it to a new Drupa node.
  chado_analysis_insert($node);

  // trim character fields
  $node->interprofile      = trim($node->interprofile);
  $node->query_uniquename  = trim($node->query_uniquename);
  $node->query_type        = trim($node->query_type);
  
  // set the type for this analysis
  tripal_analysis_insert_property($node->analysis_id, 'analysis_type', 'tripal_analysis_interpro');

  // now add in the remaining settings as a single property but separated by bars
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_interprofile', trim($node->interprofile));
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_interproparameters', trim($node->interproparameters));
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_parsego', trim($node->parsego));
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_query_re', trim($node->query_re));
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_query_type', trim($node->query_type));
  tripal_analysis_insert_property($node->analysis_id, 'analysis_interpro_query_uniquename', trim($node->query_uniquename));

  // submit the parsing jobs
  chado_analysis_interpro_submit_job($node);
}

/**
  *
  */
function chado_analysis_interpro_update($node) {

  // insert the analysistripal_core_generate_chado_var
  chado_analysis_update($node);

  // trim character fields
  $node->interprofile      = trim($node->interprofile);
  $node->query_uniquename  = trim($node->query_uniquename);
  $node->query_type        = trim($node->query_type);
  
  // set the type for this analysis
  tripal_analysis_update_property($node->analysis_id, 'analysis_type', 'tripal_analysis_interpro', 1);

  // now add in the remaining settings as a single property but separated by bars
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_interprofile', trim($node->interprofile), 1);
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_interproparameters', trim($node->interproparameters), 1);
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_parsego', trim($node->parsego), 1);
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_query_re', trim($node->query_re), 1);
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_query_type', trim($node->query_type), 1);
  tripal_analysis_update_property($node->analysis_id, 'analysis_interpro_query_uniquename', trim($node->query_uniquename), 1);

  // if this analysis uses the old style settings cvterm then remove that term
  $old = tripal_analysis_get_property($node->analysis_id, 'analysis_interpro_settings');
  if (count($old) > 0) {
    tripal_analysis_delete_property($node->analysis_id, 'analysis_interpro_settings');
  }

  // submit the parsing jobs
  chado_analysis_interpro_submit_job($node);
}

/**
 * Delete interpro anlysis
 */
function chado_analysis_interpro_delete($node) {
  chado_analysis_delete($node);
}

/**
 *
 */
function chado_analysis_interpro_submit_job($node) {
  global $user;

  // Add a job if the user wants to parse the xml output
  if ($node->interprojob) {
    $job_args[0] = $node->analysis_id;
    $job_args[1] = $node->interprofile;
    if ($node->parsego) {
      $job_args[2] = 1;
    }
    else {
      $job_args[2] = 0;
    }

    $fname = preg_replace ( "/.*\/(.*)/", "$1", $node->interprofile );
    $job_args [3] = $node->query_re;
    $job_args [4] = $node->query_type;
    $job_args [5] = $node->query_uniquename;
    tripal_add_job("Parse XML interpro: $fname", 'tripal_analysis_interpro', 'tripal_analysis_interpro_parseXMLFile', $job_args, $user->uid);
  }
}

/**
 * *****************************************************************************
 * This function customizes the view of the chado_analysis node.
 * It allows
 * us to generate the markup.
 */
function chado_analysis_interpro_view($node, $teaser = FALSE, $page = FALSE) {
  // use drupal's default node view:
  if (! $teaser) {
    $node = node_prepare ( $node, $teaser );
    // When previewing a node submitting form, it shows 'Array' instead of
    // correct date format. We need to format the date here
    $time = $node->timeexecuted;
    if (is_array ( $time )) {
      $month = $time ['month'];
      $day = $time ['day'];
      $year = $time ['year'];
      $timestamp = $year . '-' . $month . '-' . $day;
      $node->timeexecuted = $timestamp;
    }
    // When viewing a node, we need to reformat the analysisprop since we
    // separate each value with a bar |
    if (preg_match ( "/.*\|.*/", $node->interprofile )) {
      $prop_values = explode ( "|", $node->interprofile );
      $node->interprofile = $prop_values [0];
      $node->interproparameters = $prop_values [1];
    }
  }
  return $node;
}

/*******************************************************************************
 * tripal_analysis_interpro_nodeapi()
* HOOK: Implementation of hook_nodeapi()
* Display interpro results for allowed node types
*/
function tripal_analysis_interpro_node_view($node, $view_mode, $langcode) {
  switch ($node->type) {
    case 'chado_analysis_interpro':
      if ($view_mode == 'full') {
        $node->content['tripal_analysis_interpro_base'] = array(
            '#markup' => theme('tripal_analysis_interpro_base', array('node' => $node)),
            '#tripal_toc_id'    => 'base',
            '#tripal_toc_title' => 'Overview',
            '#weight' => -100,
        );
      }
      if ($view_mode == 'teaser') {
        $node->content['tripal_analysis_interpro_teaser'] = array(
            '#markup' => theme('tripal_analysis_interpro_teaser', array('node' => $node)),
        );
      }
      break;
    case 'chado_feature':
      if ($view_mode == 'full') {
        $node->content['tripal_feature_interpro_results'] = array(
          '#markup' => theme('tripal_feature_interpro_results', array('node' => $node)),
          '#tripal_toc_id'    => 'interpro',
          '#tripal_toc_title' => 'InterPro Assignments',
        );
      }
      break;
  }
}
